@@ -26,9 +26,9 @@ convert_test() {
 	shift
 
 	if [ -z "$features" ]; then
-		echo "    [TEST]   $1, btrfs defaults"
+		echo "    [TEST/conv]   $1, btrfs defaults"
 	else
-		echo "    [TEST]   $1, btrfs $features"
+		echo "    [TEST/conv]   $1, btrfs $features"
 	fi
 	nodesize=$2
 	shift 2
@@ -42,7 +42,7 @@ convert_test() {
 
 	# create a file to check btrfs-convert can convert regular file
 	# correct
-	run_check $SUDO_HELPER mount $IMAGE $TEST_MNT
+	run_check $SUDO_HELPER mount -o loop $IMAGE $TEST_MNT
 	run_check $SUDO_HELPER dd if=/dev/zero of=$TEST_MNT/test bs=$nodesize \
 		count=1 1>/dev/null 2>&1
 	run_check $SUDO_HELPER umount $TEST_MNT
@@ -52,7 +52,7 @@ convert_test() {
 }
 
 if ! [ -z "$TEST" ]; then
-	echo "    [TEST]   skipped all convert tests, TEST=$TEST"
+	echo "    [TEST/conv]   skipped all convert tests, TEST=$TEST"
 	exit 0
 fi
 
